searchNodes=[{"doc":"Provides a high-level interface for starting and managing Thousand Island server instances. Server instances exist as a process tree, which is usually started by starting this module's child spec within an existing supervision tree, or by calling start_link/1 directly. Examples A typical use of ThousandIsland might look like the following: defmodule MyApp.Supervisor do # ... other Supervisor boilerplate def init ( config ) do children = [ # ... other children as dictated by your app { ThousandIsland , port : 1234 , handler_module : MyApp.ConnectionHandler } ] Supervisor . init ( children , strategy : :one_for_one ) end end You can also start servers directly via the start_link/1 function: { :ok , pid } = ThousandIsland . start_link ( port : 1234 , handler_module : MyApp.ConnectionHandler ) Configuration A number of options are defined when starting a server. The complete list is defined by the t:options() type. Logging &amp; Telemetry As a low-level library, Thousand Island purposely does not do any inline logging of any kind. The ThousandIsland.Logging module defines a number of functions to aid in tracing connections at various log levels, and such logging can be dynamically enabled and disabled against an already running server. This logging is backed by :telemetry events internally, and if desired these events can also be hooked by your application for logging or metric purposes. The following is a complete list of events emitted by Thousand Island: [:listener, :start] : Emitted when the server successfully listens on the configured port. [:listener, :error] : Emitted when the server encounters an error listening on the configured port. [:listener, :shutdown] : Emitted when the server shuts down. [:acceptor, :start] : Emitted when an acceptor process starts up. [:acceptor, :accept] : Emitted when an acceptor process accepts a new client connection. [:acceptor, :shutdown] : Emitted when an acceptor process shuts down. [:handler, :start] : Emitted whenever a ThousandIsland.Handler process is made ready [:handler, :async_recv] : Emitted whenever a ThousandIsland.Handler process receives data asynchronously [:handler, :shutdown] : Emitted whenever a ThousandIsland.Handler process terminates [:handler, :error] : Emitted whenever a ThousandIsland.Handler process shuts down due to error [:socket, :handshake] : Emitted whenever a ThousandIsland.Socket.handshake/1 call completes. [:socket, :handshake_error] : Emitted whenever a ThousandIsland.Socket.handshake/1 call errors. [:socket, :recv] : Emitted whenever a ThousandIsland.Socket.recv/3 call completes. [:socket, :send] : Emitted whenever a ThousandIsland.Socket.send/2 call completes. [:socket, :sendfile] : Emitted whenever a ThousandIsland.Socket.sendfile/4 call completes. [:socket, :shutdown] : Emitted whenever a ThousandIsland.Socket.shutdown/2 call completes. [:socket, :close] : Emitted whenever a ThousandIsland.Socket.close/1 call completes. Where meaurements indicate a time duration they are are expressed in System :native units for performance reasons. They can be conveted to any desired time unit via System.convert_time_unit/3 .","ref":"ThousandIsland.html","title":"ThousandIsland","type":"module"},{"doc":"Returns the local port number that the servrer is listening on.","ref":"ThousandIsland.html#local_port/1","title":"ThousandIsland.local_port/1","type":"function"},{"doc":"Starts a ThousandIsland instance with the given options. Returns a pid that can be used to further manipulate the server via other functions defined on this module in the case of success, or an error tuple describing the reason the server was unable to start in the case of failure.","ref":"ThousandIsland.html#start_link/1","title":"ThousandIsland.start_link/1","type":"function"},{"doc":"Synchronously stops the given server, waiting up to the given number of milliseconds for existing connections to finish up. Immediately upon calling this function, the server stops listening for new connections, and then proceeds to wait until either all existing connections have completed or the specified timeout has elapsed.","ref":"ThousandIsland.html#stop/2","title":"ThousandIsland.stop/2","type":"function"},{"doc":"Possible options to configure a server. Valid option values are as follows: handler_module : The name of the module used to handle connections to this server. The module is expected to implement the ThousandIsland.Handler behaviour. Required. handler_options : A term which is passed as the initial state value to ThousandIsland.Handler.handle_connection/2 calls. Optional, defaulting to nil. port : The TCP port number to listen on. If not specified this defaults to 4000. If a port number of 0 is given, the server will dynamically assign a port number which can then be obtained via local_port/1 . transport_module : The name of the module which provides basic socket functions. Thousand Island provides ThousandIsland.Transports.TCP and ThousandIsland.Transports.SSL , which provide clear and TLS encrypted TCP sockets respectively. If not specified this defaults to ThousandIsland.Transports.TCP . transport_options : A keyword list of options to be passed to the transport module's ThousandIsland.Transport.listen/2 function. Valid values depend on the transport module specified in transport_module and can be found in the documentation for the ThousandIsland.Transports.TCP and ThousandIsland.Transports.SSL modules. Any options in terms of interfaces to listen to / certificates and keys to use for SSL connections will be passed in via this option. num_acceptors : The numbner of acceptor processes to run. Defaults to 10.","ref":"ThousandIsland.html#t:options/0","title":"ThousandIsland.options/0","type":"type"},{"doc":"ThousandIsland.Handler defines the behaviour required of the application layer of a Thousand Island server. When starting a Thousand Island server, you must pass the name of a module implementing this behaviour as the handler_module parameter. Thousand Island will then use the specified module to handle each connection that is made to the server. The lifecycle of a Handler instance is as follows: After a client connection to a Thousand Island server is made, Thousand Island will complete the initial setup of the connection (performing a TLS handshake, for example), and then call handle_connection/2 . A handler implementation may choose to process a client connection within the handle_connection/2 callback by calling functions against the passed ThousandIsland.Socket . In many cases, this may be all that may be required of an implementation &amp; the value {:ok, :close, state} can be returned which will cause Thousand Island to close the connection to the client. In cases where the server wishes to keep the connection open and wait for subsequent requests from the client on the same socket, it may elect to return {:ok, :continue, state} . This will cause Thousand Island to wait for client data asynchronously; handle_data/3 will be invoked when the client sends more data. In the meantime, the process which is hosting connection is idle &amp; able to receive messages sent from elsewhere in your application as needed. The implementation included in the use ThousandIsland.Handler macro uses a GenServer structure, so you may implement such behaviour via standard GenServer patterns. Note that in these cases that state is provided (and must be returned) in a {socket, state} format, where the second tuple is the same state value that is passed to the various handle_* callbacks defined on this behaviour. Note also that any GenServer handle_* calls which are processed directly by an implementing module will cancel any async read timeout values which may have been set. Such calls are able to reset the timeout by returning a four element tuple with timeout as the fourth argument as specified in the GenServer documentation. It is fully supported to intermix synchronous ThousandIsland.Socket.recv calls with async return values from handle_connection/2 and handle_data/3 callbacks. Example A simple example of a Hello World server is as follows: defmodule HelloWorld do use ThousandIsland.Handler @impl ThousandIsland.Handler def handle_connection ( socket , state ) do ThousandIsland.Socket . send ( socket , &quot;Hello, World&quot; ) { :ok , :close , state } end end Another example of a server that echoes back all data sent to it is as follows: defmodule Echo do use ThousandIsland.Handler @impl ThousandIsland.Handler def handle_data ( data , socket , state ) do ThousandIsland.Socket . send ( socket , data ) { :ok , :continue , state } end end Note that in this example there is no handle_connection/2 callback defined. The default implementation of this callback will simply return {:ok, :continue, state} , which is appropriate for cases where the client is the first party to communicate. Another example of a server which can send and receive messages asynchronously is as follows: defmodule Messenger do use ThousandIsland.Handler @impl ThousandIsland.Handler def handle_data ( msg , _socket , state ) do IO . puts ( msg ) { :ok , :continue , state } end def handle_info ( { :send , msg } , { socket , state } ) do ThousandIsland.Socket . send ( socket , msg ) { :noreply , { socket , state } } end end Note that in this example we make use of the fact that the handler process is really just a GenServer to send it messages which are able to make use of the underlying socket. This allows for bidirectional sending and receiving of messages in an asynchronous manner. When Handler Isn't Enough The use ThousandIsland.Handler implementation should be flexible enough to power just about any handler, however if this should not be the case for you, there is an escape hatch available. If you require more flexibility than the ThousandIsland.Handler behaviour provides, you are free to specify any module which implements start_link/1 as the handler_module parameter. The process of getting from this new process to a ready-to-use socket is somewhat delicate, however. The steps required are as follows: Thousand Island calls start_link/1 on the configured handler_module , passing in the configured handler_options as the sole argument. This function is expected to return a conventional GenServer.on_start() style tuple. Note that this newly created process is not passed the connection socket immediately. The socket will be passed to the new process via a message of the form {:thousand_island_ready, socket} . Once the process receives the socket, it must call ThousandIsland.Socket.handshake/1 with the socket as the sole argument in order to finalize the setup of the socket. The socket is now ready to use. In addition to this process, there are several other considerations to be aware of: The underlying socket is closed automatically when the handler process ends. Handler processes should have a restart strategy of :temporary to ensure that Thousand Island does not attempt to restart crashed handlers. Handler processes should trap exit if possible so that existing connections can be given a chance to cleanly shut down when shutting down a Thousand Island server instance. The :handler family of telemetry events are emitted by the ThousandIsland.Handler implementation. If you use your own implementation in its place you will not see any such telemetry events.","ref":"ThousandIsland.Handler.html","title":"ThousandIsland.Handler","type":"behaviour"},{"doc":"This callback is called when the underlying socket is closed by the remote end; it should perform any cleanup required as it is the last callback called before the process backing this connection is terminated. The underlying socket has already been closed by the time this callback is called. The return value is ignored. This callback is not called if the connection is explicitly closed via ThousandIsland.Socket.close/1 , however it will be called in cases where handle_connection/2 or handle_data/3 return a {:ok, :close, state} tuple.","ref":"ThousandIsland.Handler.html#c:handle_close/2","title":"ThousandIsland.Handler.handle_close/2","type":"callback"},{"doc":"This callback is called shortly after a client connection has been made, immediately after the socket handshake process has completed. It is called with the server's configured handler_options value as initial state. Handlers may choose to interact synchronously with the socket in this callback via calls to various ThousandIsland.Socket functions. The value returned by this callback causes Thousand Island to proceed in once of several ways: Returning {:ok, :close, state} will cause Thousand Island to close the socket &amp; call the handle_close/2 callback to allow final cleanup to be done. Returning {:ok, :continue, state} will cause Thousand Island to switch the socket to an asynchronous mode. When the client subsequently sends data (or if there is already unread data waiting from the client), Thousand Island will call handle_data/3 to allow this data to be processed. Returning {:ok, :continue, state, timeout} is identical to the previous case with the addition of a timeout. If timeout milliseconds passes with no data being received, the socket will be closed and handle_timeout/2 will be called. Returning {:error, reason, state} will cause Thousand Island to close the socket &amp; call the handle_error/3 callback to allow final cleanup to be done.","ref":"ThousandIsland.Handler.html#c:handle_connection/2","title":"ThousandIsland.Handler.handle_connection/2","type":"callback"},{"doc":"This callback is called whenever client data is received after handle_connection/2 or handle_data/3 have returned an {:ok, :continue, state} tuple. The data received is passed as the first argument, and handlers may choose to interact synchronously with the socket in this callback via calls to various ThousandIsland.Socket functions. The value returned by this callback causes Thousand Island to proceed in once of several ways: Returning {:ok, :close, state} will cause Thousand Island to close the socket &amp; call the handle_close/2 callback to allow final cleanup to be done. Returning {:ok, :continue, state} will cause Thousand Island to switch the socket to an asynchronous mode. When the client subsequently sends data (or if there is already unread data waiting from the client), Thousand Island will call handle_data/3 to allow this data to be processed. Returning {:ok, :continue, state, timeout} is identical to the previous case with the addition of a timeout. If timeout milliseconds passes with no data being received, the socket will be closed and handle_timeout/2 will be called. Returning {:error, reason, state} will cause Thousand Island to close the socket &amp; call the handle_error/3 callback to allow final cleanup to be done.","ref":"ThousandIsland.Handler.html#c:handle_data/3","title":"ThousandIsland.Handler.handle_data/3","type":"callback"},{"doc":"This callback is called when the underlying socket encounters an error; it should perform any cleanup required as it is the last callback called before the process backing this connection is terminated. The underlying socket has already been closed by the time this callback is called. The return value is ignored. In addition to socket level errors, this callback is also called in cases where handle_connection/2 or handle_data/3 return a {:error, reason, state} tuple.","ref":"ThousandIsland.Handler.html#c:handle_error/3","title":"ThousandIsland.Handler.handle_error/3","type":"callback"},{"doc":"This callback is called when the server process itself is being shut down; it should perform any cleanup required as it is the last callback called before the process backing this connection is terminated. The underlying socket has NOT been closed by the time this callback is called. The return value is ignored. This callback is only called when the shutdown reason is :normal , and is subject to the same caveats described in GenServer.terminate/2 .","ref":"ThousandIsland.Handler.html#c:handle_shutdown/2","title":"ThousandIsland.Handler.handle_shutdown/2","type":"callback"},{"doc":"This callback is called when an async read call times out (ie: when a tuple of the form {:ok, :continue, state, timeout} is returned by handle_connection/2 or handle_data/3 and timeout ms have passed). Note that it is NOT called on explicit ThousandIsland.Socket.recv/3 calls as they have their own timeout semantics. The underlying socket has NOT been closed by the time this callback is called. The return value is ignored.","ref":"ThousandIsland.Handler.html#c:handle_timeout/2","title":"ThousandIsland.Handler.handle_timeout/2","type":"callback"},{"doc":"The value returned by handle_connection/2 and handle_data/3","ref":"ThousandIsland.Handler.html#t:handler_result/0","title":"ThousandIsland.Handler.handler_result/0","type":"type"},{"doc":"Allows dynamically adding and altering the log level used to trace connections within a Thousand Island server via the use of telemetry hooks. Should you wish to do your own logging or tracking of these events, a complete list of the telemetry events emitted by Thousand Island is described in the module documentation for ThousandIsland .","ref":"ThousandIsland.Logger.html","title":"ThousandIsland.Logger","type":"module"},{"doc":"Start logging Thousand Island at the specified log level. Valid values for log level are :error , :info , :debug , and :trace . Enabling a given log level implicitly enables all higher log levels as well.","ref":"ThousandIsland.Logger.html#attach_logger/1","title":"ThousandIsland.Logger.attach_logger/1","type":"function"},{"doc":"Stop logging Thousand Island at the specified log level. Disabling a given log level implicitly disables all lower log levels as well.","ref":"ThousandIsland.Logger.html#detach_logger/1","title":"ThousandIsland.Logger.detach_logger/1","type":"function"},{"doc":"","ref":"ThousandIsland.Logger.html#log_debug/4","title":"ThousandIsland.Logger.log_debug/4","type":"function"},{"doc":"","ref":"ThousandIsland.Logger.html#log_info/4","title":"ThousandIsland.Logger.log_info/4","type":"function"},{"doc":"","ref":"ThousandIsland.Logger.html#log_trace/4","title":"ThousandIsland.Logger.log_trace/4","type":"function"},{"doc":"Encapsulates a client connection's underlying socket, providing a facility to read, write, and otherwise manipulate a connection from a client.","ref":"ThousandIsland.Socket.html","title":"ThousandIsland.Socket","type":"module"},{"doc":"Closes the given socket. Note that a socket is automatically closed when the handler process which owns it terminates","ref":"ThousandIsland.Socket.html#close/1","title":"ThousandIsland.Socket.close/1","type":"function"},{"doc":"Handshakes the underlying socket if it is required (as in the case of SSL sockets, for example).","ref":"ThousandIsland.Socket.html#handshake/1","title":"ThousandIsland.Socket.handshake/1","type":"function"},{"doc":"Returns information in the form of t:ThousandIsland.Transport.socket_info() about the local end of the socket.","ref":"ThousandIsland.Socket.html#local_info/1","title":"ThousandIsland.Socket.local_info/1","type":"function"},{"doc":"Returns information about the protocol negotiated during transport handshaking (if any).","ref":"ThousandIsland.Socket.html#negotiated_protocol/1","title":"ThousandIsland.Socket.negotiated_protocol/1","type":"function"},{"doc":"Returns information in the form of t:ThousandIsland.Transport.socket_info() about the remote end of the socket.","ref":"ThousandIsland.Socket.html#peer_info/1","title":"ThousandIsland.Socket.peer_info/1","type":"function"},{"doc":"Returns available bytes on the given socket. Up to num_bytes bytes will be returned (0 can be passed in to get the next 'available' bytes, typically the next packet). If insufficient bytes are available, the function can wait timeout milliseconds for data to arrive.","ref":"ThousandIsland.Socket.html#recv/3","title":"ThousandIsland.Socket.recv/3","type":"function"},{"doc":"Sends the given data (specified as a binary or an IO list) on the given socket.","ref":"ThousandIsland.Socket.html#send/2","title":"ThousandIsland.Socket.send/2","type":"function"},{"doc":"Sends the contents of the given file based on the provided offset &amp; length","ref":"ThousandIsland.Socket.html#sendfile/4","title":"ThousandIsland.Socket.sendfile/4","type":"function"},{"doc":"Sets the given flags on the socket","ref":"ThousandIsland.Socket.html#setopts/2","title":"ThousandIsland.Socket.setopts/2","type":"function"},{"doc":"Shuts down the socket in the given direction.","ref":"ThousandIsland.Socket.html#shutdown/2","title":"ThousandIsland.Socket.shutdown/2","type":"function"},{"doc":"A reference to a socket along with metadata describing how to use it","ref":"ThousandIsland.Socket.html#t:t/0","title":"ThousandIsland.Socket.t/0","type":"type"},{"doc":"This module describes the behaviour required for Thousand Island to interact with low-level sockets. It is largely internal to Thousand Island, however users are free to implement their own versions of this behaviour backed by whatever underlying transport they choose. Such a module can be used in Thousand Island by passing its name as the transport_module option when starting up a server, as described in ThousandIsland .","ref":"ThousandIsland.Transport.html","title":"ThousandIsland.Transport","type":"behaviour"},{"doc":"Wait for a client connection on the given listener socket. This call blocks until such a connection arrives, or an error occurs (such as the listener socket being closed).","ref":"ThousandIsland.Transport.html#c:accept/1","title":"ThousandIsland.Transport.accept/1","type":"callback"},{"doc":"Closes the given socket.","ref":"ThousandIsland.Transport.html#c:close/1","title":"ThousandIsland.Transport.close/1","type":"callback"},{"doc":"Transfers ownership of the given socket to the given process. This will always be called by the process which currently owns the socket.","ref":"ThousandIsland.Transport.html#c:controlling_process/2","title":"ThousandIsland.Transport.controlling_process/2","type":"callback"},{"doc":"Returns stats about the connection on the socket.","ref":"ThousandIsland.Transport.html#c:getstat/1","title":"ThousandIsland.Transport.getstat/1","type":"callback"},{"doc":"Performs an initial handshake on a new client connection (such as that done when negotiating an SSL connection). Transports which do not have such a handshake can simply pass the socket through unchanged.","ref":"ThousandIsland.Transport.html#c:handshake/1","title":"ThousandIsland.Transport.handshake/1","type":"callback"},{"doc":"Create and return a listener socket bound to the given port and configured per the provided options.","ref":"ThousandIsland.Transport.html#c:listen/2","title":"ThousandIsland.Transport.listen/2","type":"callback"},{"doc":"Return the local port number that the given listener socket is accepting connections on.","ref":"ThousandIsland.Transport.html#c:listen_port/1","title":"ThousandIsland.Transport.listen_port/1","type":"callback"},{"doc":"Returns information in the form of t:socket_info() about the local end of the socket.","ref":"ThousandIsland.Transport.html#c:local_info/1","title":"ThousandIsland.Transport.local_info/1","type":"callback"},{"doc":"Returns the protocol negotiated as part of handshaking. Most typically this is via TLS' ALPN or NPN extensions. If the underlying transport does not support protocol negotiation (or if one was not negotiated), {:error, :protocol_not_negotiated} is returned","ref":"ThousandIsland.Transport.html#c:negotiated_protocol/1","title":"ThousandIsland.Transport.negotiated_protocol/1","type":"callback"},{"doc":"Returns information in the form of t:socket_info() about the remote end of the socket.","ref":"ThousandIsland.Transport.html#c:peer_info/1","title":"ThousandIsland.Transport.peer_info/1","type":"callback"},{"doc":"Returns available bytes on the given socket. Up to num_bytes bytes will be returned (0 can be passed in to get the next 'available' bytes, typically the next packet). If insufficient bytes are available, the function can wait timeout milliseconds for data to arrive.","ref":"ThousandIsland.Transport.html#c:recv/3","title":"ThousandIsland.Transport.recv/3","type":"callback"},{"doc":"Sends the given data (specified as a binary or an IO list) on the given socket.","ref":"ThousandIsland.Transport.html#c:send/2","title":"ThousandIsland.Transport.send/2","type":"callback"},{"doc":"Sends the contents of the given file based on the provided offset &amp; length","ref":"ThousandIsland.Transport.html#c:sendfile/4","title":"ThousandIsland.Transport.sendfile/4","type":"callback"},{"doc":"Sets the given options on the socket. Should disallow setting of options which are not compatible with Thousand Island","ref":"ThousandIsland.Transport.html#c:setopts/2","title":"ThousandIsland.Transport.setopts/2","type":"callback"},{"doc":"Shuts down the socket in the given direction.","ref":"ThousandIsland.Transport.html#c:shutdown/2","title":"ThousandIsland.Transport.shutdown/2","type":"callback"},{"doc":"A listener socket used to wait for connections","ref":"ThousandIsland.Transport.html#t:listener_socket/0","title":"ThousandIsland.Transport.listener_socket/0","type":"type"},{"doc":"The return value from a negotiated_protocol/1 call","ref":"ThousandIsland.Transport.html#t:negotiated_protocol_info/0","title":"ThousandIsland.Transport.negotiated_protocol_info/0","type":"type"},{"doc":"The return value from a handshake/1 call","ref":"ThousandIsland.Transport.html#t:on_handshake/0","title":"ThousandIsland.Transport.on_handshake/0","type":"type"},{"doc":"The return value from a recv/3 call","ref":"ThousandIsland.Transport.html#t:on_recv/0","title":"ThousandIsland.Transport.on_recv/0","type":"type"},{"doc":"A socket representing a client connection","ref":"ThousandIsland.Transport.html#t:socket/0","title":"ThousandIsland.Transport.socket/0","type":"type"},{"doc":"Information about an endpoint (either remote ('peer') or local","ref":"ThousandIsland.Transport.html#t:socket_info/0","title":"ThousandIsland.Transport.socket_info/0","type":"type"},{"doc":"Options which can be set on a socket via setopts/2","ref":"ThousandIsland.Transport.html#t:socket_options/0","title":"ThousandIsland.Transport.socket_options/0","type":"type"},{"doc":"Connection statistics for a given socket","ref":"ThousandIsland.Transport.html#t:socket_stats/0","title":"ThousandIsland.Transport.socket_stats/0","type":"type"},{"doc":"The direction in which to shutdown a connection in advance of closing it","ref":"ThousandIsland.Transport.html#t:way/0","title":"ThousandIsland.Transport.way/0","type":"type"},{"doc":"Defines a ThousandIsland.Transport implementation based on TCP SSL sockets as provided by Erlang's :ssl module. For the most part, users of Thousand Island will only ever need to deal with this module via transport_options passed to ThousandIsland at startup time. A complete list of such options is defined via the t::ssl.tls_server_option type. This list can be somewhat difficult to decipher; a list of the most common options follows: key : A DER encoded binary representation of the SSL key to use cert : A DER encoded binary representation of the SSL key to use keyfile : A string path to a PEM encoded key to use for SSL certfile : A string path to a PEM encoded cert to use for SSL ip : The IP to listen on (defaults to all interfaces). IPs should be described in tuple form (ie: ip: {1, 2, 3, 4} ). The value :loopback can be used to only bind to localhost. On platforms which support it (macOS and Linux at a minimum, likely others), you can also bind to a Unix domain socket by specifying a value of ip: {:local, &quot;/path/to/socket&quot;} . Note that the port must be set to 0 , and that the socket is not removed from the filesystem after the server shuts down. Unless overridden, this module uses the following default options: backlog : 1024 , nodelay : true , linger : { true , 30 } , send_timeout : 30_000 , send_timeout_close : true , reuseaddr : true The following options are required for the proper operation of Thousand Island and cannot be overridden at startup (though they can be set via calls to setopts/2 ) mode : :binary , active : false","ref":"ThousandIsland.Transports.SSL.html","title":"ThousandIsland.Transports.SSL","type":"module"},{"doc":"Defines a ThousandIsland.Transport implementation based on clear TCP sockets as provided by Erlang's :gen_tcp module. For the most part, users of Thousand Island will only ever need to deal with this module via transport_options passed to ThousandIsland at startup time. A complete list of such options is defined via the t::gen_tcp.listen_option() type. This list can be somewhat difficult to decipher; by far the most common value to pass to this transport is the following: ip : The IP to listen on (defaults to all interfaces). IPs should be described in tuple form (ie: ip: {1, 2, 3, 4} ). The value :loopback can be used to only bind to localhost. On platforms which support it (macOS and Linux at a minimum, likely others), you can also bind to a Unix domain socket by specifying a value of ip: {:local, &quot;/path/to/socket&quot;} . Note that the port must be set to 0 , and that the socket is not removed from the filesystem after the server shuts down. Unless overridden, this module uses the following default options: backlog : 1024 , nodelay : true , linger : { true , 30 } , send_timeout : 30_000 , send_timeout_close : true , reuseaddr : true The following options are required for the proper operation of Thousand Island and cannot be overridden at startup (though they can be set via calls to setopts/2 ) mode : :binary , active : false","ref":"ThousandIsland.Transports.TCP.html","title":"ThousandIsland.Transports.TCP","type":"module"}]